from django.urls import path, include
from django.contrib.admin import site
from django.contrib.auth import views as auth_views
from django.conf import settings

from kmuhelper import admin
from kmuhelper import views
from kmuhelper import startup

#######################

app_name = 'kmuhelper'
urlpatterns = [
    path('',        views.home, name="home"),
    path('admin/',  views.admin, name="admin"),

    path('_templatetest/<path:templatename>', views._templatetest, name="_templatetest"),

    path('',        include('kmuhelper.main')),
    path('api/',    include('kmuhelper.api')),
    path('app/',    include('kmuhelper.app')),
    path('emails/', include('kmuhelper.emails')),
    path('stats/',  include('kmuhelper.stats')),
    path('wc/',     include('kmuhelper.integrations.woocommerce')),
]



from django.contrib import admin

# Disable "view on site" globally (for other apps too!)

admin.site.site_url = None

# Register your models here.

from kmuhelper.app import admin as appadmin
from kmuhelper.main import admin as mainadmin
from kmuhelper.emails import admin as emailadmin

# Custom Admin

from django.urls import path, include

modeladmins = appadmin.modeladmins + mainadmin.modeladmins + emailadmin.modeladmins



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseBadRequest, JsonResponse
from django.shortcuts import render, redirect
from django.urls import reverse, path, reverse_lazy
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe
from django.db import models

from kmuhelper.utils import package_version, python_version
from kmuhelper.models import Bestellung

#####

def not_authenticated(request):
    return JsonResponse({
        "error": "Not authenticated!"
    })

#####

@login_required(login_url=reverse_lazy("kmuhelper:api-not-authenticated"))
def versions(request):
    context = {
        "versions": {
            "Python": {"current": python_version(), "latest": "-", "uptodate": None},
        }
    }

    packages = ["Django", "django-kmuhelper", "gunicorn",
                "requests", "WooCommerce", "reportlab", "rich", "pytz"]

    for package in packages:
        context["versions"][package] = package_version(package)
    return JsonResponse(context)


@login_required(login_url=reverse_lazy("kmuhelper:api-not-authenticated"))
def orders_unpaid(request):
    sum_unsent = Bestellung.objects.filter(bezahlt=False, versendet=False).aggregate(
        models.Sum('fix_summe'))["fix_summe__sum"]
    sum_sent = Bestellung.objects.filter(bezahlt=False, versendet=True).aggregate(
        models.Sum('fix_summe'))["fix_summe__sum"]

    context = {
        "orders_unpaid_sum": {
            "unsent": round(sum_unsent, 2),
            "sent": round(sum_sent, 2),
            "all": round(sum_unsent, 2)+round(sum_sent, 2)
        }
    }
    return JsonResponse(context)



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseBadRequest, JsonResponse
from django.shortcuts import render, redirect
from django.urls import reverse, path, reverse_lazy
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe

from kmuhelper.utils import package_version, python_version

from kmuhelper.emails.models import EMail

#####

def email_view(request, object_id):
    if EMail.objects.filter(id=object_id).exists():
        email = EMail.objects.get(id=object_id)
        t1 = request.GET.get("token", None)
        t2 = str(email.token)
        if request.user.has_perm("kmuhelper.view_email") or (t1 == t2):
            return HttpResponse(email.render(online=True))
        else:
            return HttpResponseBadRequest("Du hast keinen Zugriff auf diese E-Mail!")
    else:
        return HttpResponseBadRequest("E-Mail wurde nicht gefunden!")



# pylint: disable=no-member

from django.utils.html import strip_tags

from woocommerce import API as WCAPI

from kmuhelper.main.models import Einstellung, Geheime_Einstellung, Produkt, Kunde, Kategorie, Bestellung, Bestellungsposten, Kosten
from kmuhelper.utils import runden

###############

from rich import print
from rich.progress import Progress

prefix = "[deep_pink4][KMUHelper][/] -"


def log(string, *args):
    print(prefix, string, *args)


def preparestring(string):
    return strip_tags(string.replace("</p>", " ").replace("</strong>", " ")).replace("&#8211;", "-").replace("&#8211;", " ").replace("&#215;", "x").replace("&#8220;", '"').replace("&#8221;", '"').replace("&nbsp;", " ").replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&").replace("&quot;", '"').replace("&#8222;", '"').replace("  ", " ")

###############


class WooCommerce():
    @classmethod
    def get_api(self):
        return WCAPI(
            url=Geheime_Einstellung.objects.get(id="wc-url").get_inhalt(),
            consumer_key=Geheime_Einstellung.objects.get(
                id="wc-consumer_key").get_inhalt(),
            consumer_secret=Geheime_Einstellung.objects.get(
                id="wc-consumer_secret").get_inhalt()
        )

    @classmethod
    def product_create(self, product):
        try:
            verkaufspreis = float(product["price"])
        except ValueError:
            verkaufspreis = 0.0

        newproduct = Produkt.objects.create(
            woocommerceid=product["id"],
            artikelnummer=product["sku"],
            name=preparestring(product["name"]),
            kurzbeschrieb=preparestring(product["short_description"]),
            beschrieb=preparestring(product["description"]),
            verkaufspreis=verkaufspreis,
            bildlink=(product["images"][0]["src"]) if len(
                product["images"]) > 0 else "",
            aktion_von=(product["date_on_sale_from_gmt"] +
                        "+00:00" if product["date_on_sale_from"] else None),
            aktion_bis=(product["date_on_sale_to_gmt"] +
                        "+00:00" if product["date_on_sale_to"] else None),
            aktion_preis=(product["sale_price"]
                          if product["sale_price"] else None)
        )
        if product["manage_stock"]:
            newproduct.lagerbestand = product["stock_quantity"]
        for category in product["categories"]:
            tup = Kategorie.objects.get_or_create(woocommerceid=category["id"])
            if tup[1]:
                self.category_update(tup[0])
            newproduct.kategorien.add(tup[0])
        newproduct.save()
        log("Produkt erstellt:", str(newproduct))
        return newproduct

    @classmethod
    def product_update(self, product, newproduct=None):
        if not newproduct:
            newproduct = self.get_api().get("products/" + str(product.woocommerceid)).json()

        try:
            product.verkaufspreis = float(newproduct["price"])
        except ValueError:
            pass
        except KeyError:
            if "code" in newproduct and newproduct["code"] == "woocommerce_rest_product_invalid_id":
                log(prefix +
                    " [red]Produkt existiert in WooCommerce nicht![/]")
                product.woocommerceid = 0
                product.save()
                return product

        product.artikelnummer = newproduct["sku"]
        product.name = preparestring(newproduct["name"])
        product.kurzbeschrieb = preparestring(newproduct["short_description"])
        product.beschrieb = preparestring(newproduct["description"])
        product.bildlink = (newproduct["images"][0]["src"]) if len(
            newproduct["images"]) > 0 else ""
        if newproduct["date_on_sale_from_gmt"]:
            product.aktion_von = newproduct["date_on_sale_from_gmt"] + "+00:00"
        if newproduct["date_on_sale_to_gmt"]:
            product.aktion_bis = newproduct["date_on_sale_to_gmt"] + "+00:00"
        if newproduct["sale_price"]:
            product.aktion_preis = newproduct["sale_price"]

        product.kategorien.clear()
        newcategories = []
        for category in newproduct["categories"]:
            tup = Kategorie.objects.get_or_create(woocommerceid=category["id"])
            if tup[1]:
                self.category_update(tup[0])
            newcategories.append(tup[0])
        product.kategorien.add(*newcategories)

        product.save()
        log("Produkt aktualisiert:", str(product))
        return product

    @classmethod
    def product_import(self):
        with Progress() as progress:
            task_prepare = progress.add_task(
                prefix + " [orange_red1]Produktdownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Produkt.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            wcapi = self.get_api()
            productlist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                prefix + " [green]Produkte herunterladen...")

            r = wcapi.get("products?exclude=" + excludeids)
            productlist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                productlist += wcapi.get("products?exclude=" +
                                         excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)

            progress.stop_task(task_download)

            if productlist:
                task_process = progress.add_task(
                    prefix + " [cyan]Produkte verarbeiten...", total=len(productlist))
                for product in productlist:
                    self.product_create(product)
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)
        return len(productlist)

    @classmethod
    def product_bulk_update(self, products):
        with Progress() as progress:
            task = progress.add_task(
                prefix + " [orange_red1]Produkte aktualisieren...", total=products.count())
            successcount = 0
            errorcount = 0
            for product in products:
                if product.woocommerceid:
                    self.product_update(product)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)

    @classmethod
    def customer_create(self, customer):
        newcustomer = Kunde.objects.create(
            woocommerceid=customer["id"],

            email=customer["email"],
            vorname=customer["first_name"],
            nachname=customer["last_name"],
            firma=customer["billing"]["company"],
            benutzername=customer["username"],
            avatar_url=customer["avatar_url"],

            rechnungsadresse_vorname=customer["billing"]["first_name"],
            rechnungsadresse_nachname=customer["billing"]["last_name"],
            rechnungsadresse_firma=customer["billing"]["company"],
            rechnungsadresse_adresszeile1=customer["billing"]["address_1"],
            rechnungsadresse_adresszeile2=customer["billing"]["address_2"],
            rechnungsadresse_ort=customer["billing"]["city"],
            rechnungsadresse_kanton=customer["billing"]["state"],
            rechnungsadresse_plz=customer["billing"]["postcode"],
            rechnungsadresse_land=customer["billing"]["country"],
            rechnungsadresse_email=customer["billing"]["email"],
            rechnungsadresse_telefon=customer["billing"]["phone"],

            lieferadresse_vorname=customer["shipping"]["first_name"],
            lieferadresse_nachname=customer["shipping"]["last_name"],
            lieferadresse_firma=customer["shipping"]["company"],
            lieferadresse_adresszeile1=customer["shipping"]["address_1"],
            lieferadresse_adresszeile2=customer["shipping"]["address_2"],
            lieferadresse_ort=customer["shipping"]["city"],
            lieferadresse_kanton=customer["shipping"]["state"],
            lieferadresse_plz=customer["shipping"]["postcode"],
            lieferadresse_land=customer["shipping"]["country"],
        )
        log("Kunde erstellt:", str(newcustomer))
        return newcustomer

    @classmethod
    def customer_update(self, customer, newcustomer=None):
        if not newcustomer:
            newcustomer = self.get_api().get("customers/" + str(customer.woocommerceid)).json()

        if "code" in newcustomer and newcustomer["code"] == "woocommerce_rest_customer_invalid_id":
            log("[red]Kunde existiert in WooCommerce nicht![/]")
            customer.woocommerceid = 0
            customer.save()
            return customer

        customer.email = newcustomer["email"]
        customer.vorname = newcustomer["first_name"]
        customer.nachname = newcustomer["last_name"]
        customer.firma = newcustomer["billing"]["company"] if newcustomer["billing"]["company"] else customer.firma
        customer.benutzername = newcustomer["username"]
        customer.avatar_url = newcustomer["avatar_url"]

        customer.rechnungsadresse_vorname = newcustomer["billing"]["first_name"]
        customer.rechnungsadresse_nachname = newcustomer["billing"]["last_name"]
        customer.rechnungsadresse_firma = newcustomer["billing"]["company"]
        customer.rechnungsadresse_adresszeile1 = newcustomer["billing"]["address_1"]
        customer.rechnungsadresse_adresszeile2 = newcustomer["billing"]["address_2"]
        customer.rechnungsadresse_ort = newcustomer["billing"]["city"]
        customer.rechnungsadresse_kanton = newcustomer["billing"]["state"]
        customer.rechnungsadresse_plz = newcustomer["billing"]["postcode"]
        customer.rechnungsadresse_land = newcustomer["billing"]["country"]
        customer.rechnungsadresse_email = newcustomer["billing"]["email"]
        customer.rechnungsadresse_telefon = newcustomer["billing"]["phone"]

        customer.lieferadresse_vorname = newcustomer["shipping"]["first_name"]
        customer.lieferadresse_nachname = newcustomer["shipping"]["last_name"]
        customer.lieferadresse_firma = newcustomer["shipping"]["company"]
        customer.lieferadresse_adresszeile1 = newcustomer["shipping"]["address_1"]
        customer.lieferadresse_adresszeile2 = newcustomer["shipping"]["address_2"]
        customer.lieferadresse_ort = newcustomer["shipping"]["city"]
        customer.lieferadresse_kanton = newcustomer["shipping"]["state"]
        customer.lieferadresse_plz = newcustomer["shipping"]["postcode"]
        customer.lieferadresse_land = newcustomer["shipping"]["country"]
        customer.save()
        log("Kunde aktualisiert:", str(customer))
        return customer

    @classmethod
    def customer_import(self):
        with Progress() as progress:
            task_prepare = progress.add_task(
                prefix + " [orange_red1]Kundendownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Kunde.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            wcapi = self.get_api()
            customerlist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                prefix + " [green]Kunden herunterladen...")

            r = wcapi.get("customers?exclude=" + excludeids)
            customerlist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                customerlist += wcapi.get("customers?exclude=" +
                                          excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)
            progress.stop_task(task_download)

            if customerlist:
                task_process = progress.add_task(
                    prefix + " [cyan]Kunden verarbeiten...", total=len(customerlist))
                for customer in customerlist:
                    self.customer_create(customer)
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)
        return len(customerlist)

    @classmethod
    def customer_bulk_update(self, customers):
        with Progress() as progress:
            task = progress.add_task(
                prefix + " [orange_red1]Kunden aktualisieren...", total=customers.count())
            successcount = 0
            errorcount = 0
            for customer in customers:
                if customer.woocommerceid:
                    self.customer_update(customer)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)

    @classmethod
    def category_update(self, category, newcategory=None):
        if not newcategory:
            newcategory = self.get_api().get("products/categories/" +
                                             str(category.woocommerceid)).json()

        if "code" in newcategory and newcategory["code"] == "woocommerce_rest_category_invalid_id":
            log("[red]Kategorie existiert in WooCommerce nicht![/]")
            category.woocommerceid = 0
            category.save()
            return category

        category.name = preparestring(newcategory["name"])
        category.beschrieb = preparestring(newcategory["description"])
        category.bildlink = (
            newcategory["image"]["src"]) if newcategory["image"] else ""
        if newcategory["parent"]:
            tup = Kategorie.objects.get_or_create(
                woocommerceid=newcategory["parent"])
            if tup[1]:
                self.category_update(tup[0])
            category.uebergeordnete_kategorie = tup[0]
        category.save()
        log("Kategorie aktualisiert:", str(category))
        return category

    @classmethod
    def category_import(self):
        # excludeids = str([obj.woocommerceid for obj in Kategorie.objects.all().exclude(
        #     woocommerceid=0)])[1:-1]
        # wcapi = self.get_api()
        # log("Kategorien von WooCommerce herunterladen...")
        # categorylist = []
        # r = wcapi.get("products/categories?exclude=" + excludeids)
        # categorylist += r.json()
        # for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
        #     categorylist += wcapi.get("products/categories?exclude=" +
        #                               excludeids + "&page=" + str(page)).json()
        # log("Kategorien von WooCommerce heruntergeladen:",
        #     len(categorylist))
        # log("Kategorien von WooCommerce importieren...")
        # categorieswithparents = []
        # for category in categorylist:
        #     newcategory = Kategorie.objects.create(
        #         name=preparestring(category["name"]),
        #         beschrieb=preparestring(category["description"]),
        #         bildlink=(category["image"]["src"]
        #                   ) if category["image"] else "",
        #         woocommerceid=category["id"]
        #     )
        #     log("Kategorie erstellt:", category["name"])
        #     if category["parent"]:
        #         categorieswithparents.append((newcategory, category["parent"]))
        # for tup in categorieswithparents:
        #     tup[0].uebergeordnete_kategorie = Kategorie.objects.get(
        #         woocommerceid=tup[1])
        #     tup[0].save()
        # log("Kategorien von WooCommerce importiert:",
        #     len(categorylist))
        # return len(categorylist)

        with Progress() as progress:
            task_prepare = progress.add_task(
                prefix + " [orange_red1]Kategoriendownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Kategorie.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            wcapi = self.get_api()
            categorylist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                prefix + " [green]Kategorien herunterladen...")

            r = wcapi.get("products/categories?exclude=" + excludeids)
            categorylist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                categorylist += wcapi.get("products/categories?exclude=" +
                                          excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)

            progress.stop_task(task_download)

            categorieswithparents = []

            if categorylist:
                task_process = progress.add_task(
                    prefix + " [cyan]Kategorien verarbeiten...", total=len(categorylist))
                for category in categorylist:
                    newcategory = Kategorie.objects.create(
                        name=preparestring(category["name"]),
                        beschrieb=preparestring(category["description"]),
                        bildlink=(category["image"]["src"]
                                  ) if category["image"] else "",
                        woocommerceid=category["id"]
                    )
                    log("Kategorie erstellt:", category["name"])
                    if category["parent"]:
                        categorieswithparents.append(
                            (newcategory, category["parent"]))
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)

            if categorieswithparents:
                task_process2 = progress.add_task(
                    prefix + " [cyan]Kategorieabhängigkeiten verarbeiten...", total=len(categorieswithparents))
                for tup in categorieswithparents:
                    tup[0].uebergeordnete_kategorie = Kategorie.objects.get(
                        woocommerceid=tup[1])
                    tup[0].save()
                    progress.update(task_process2, advance=1)
                progress.stop_task(task_process2)
        return len(categorylist)

    @classmethod
    def category_bulk_update(self, categories):
        with Progress() as progress:
            task = progress.add_task(
                prefix + " [orange_red1]Kategorien aktualisieren...", total=categories.count())
            successcount = 0
            errorcount = 0
            for category in categories:
                if category.woocommerceid:
                    self.category_update(category)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)

    @classmethod
    def order_create(self, order):
        neworder = Bestellung.objects.create(
            woocommerceid=order["id"],

            status=order["status"],
            versendet=(True if order["status"] == "completed" else False),

            ausgelagert=(True if order["status"] == "completed" else False),

            zahlungsmethode=order["payment_method"],
            bezahlt=(True if order["date_paid"] else False),

            kundennotiz=order["customer_note"],

            order_key=order["order_key"],

            rechnungsadresse_vorname=order["billing"]["first_name"],
            rechnungsadresse_nachname=order["billing"]["last_name"],
            rechnungsadresse_firma=order["billing"]["company"],
            rechnungsadresse_adresszeile1=order["billing"]["address_1"],
            rechnungsadresse_adresszeile2=order["billing"]["address_2"],
            rechnungsadresse_ort=order["billing"]["city"],
            rechnungsadresse_kanton=order["billing"]["state"],
            rechnungsadresse_plz=order["billing"]["postcode"],
            rechnungsadresse_land=order["billing"]["country"],
            rechnungsadresse_email=order["billing"]["email"],
            rechnungsadresse_telefon=order["billing"]["phone"],

            lieferadresse_vorname=order["shipping"]["first_name"],
            lieferadresse_nachname=order["shipping"]["last_name"],
            lieferadresse_firma=order["shipping"]["company"],
            lieferadresse_adresszeile1=order["shipping"]["address_1"],
            lieferadresse_adresszeile2=order["shipping"]["address_2"],
            lieferadresse_ort=order["shipping"]["city"],
            lieferadresse_kanton=order["shipping"]["state"],
            lieferadresse_plz=order["shipping"]["postcode"],
            lieferadresse_land=order["shipping"]["country"]
        )
        neworder.datum = order["date_created_gmt"] + "+00:00"
        if order["customer_id"]:
            kunde = Kunde.objects.get_or_create(
                woocommerceid=int(order["customer_id"]))
            if kunde[1]:
                kunde = self.customer_update(kunde[0])
            else:
                kunde = kunde[0]
        else:
            kunde = None
        neworder.kunde = kunde
        for item in order["line_items"]:
            product = Produkt.objects.get_or_create(
                woocommerceid=int(item["product_id"]))
            if product[1]:
                product = self.product_update(product[0])
            else:
                product = product[0]
            neworder.produkte.add(product, through_defaults={"menge": int(
                item["quantity"]), "produktpreis": runden(float(item["price"]))})
        for item in order["shipping_lines"]:
            neworder.kosten.add(Kosten.objects.get_or_create(name=item["method_title"], preis=float(
                item["total"]), mwstsatz=(7.7 if float(item["total_tax"]) > 0 else 0))[0])
        neworder.save()
        log("Bestellung erstellt:", str(neworder))

        neworder.email_stock_warning()
        return neworder

    @classmethod
    def order_update(self, order, neworder=None):
        if not neworder:
            neworder = self.get_api().get("orders/" + str(order.woocommerceid)).json()

        if "code" in neworder and neworder["code"] == "woocommerce_rest_order_invalid_id":
            log("[red]Bestellung existiert in WooCommerce nicht![/]")
            order.woocommerceid = 0
            order.save()
            return order

        if neworder["date_paid"]:
            order.bezahlt = True
        order.status = neworder["status"]
        order.kundennotiz = neworder["customer_note"]

        order.rechnungsadresse_vorname = neworder["billing"]["first_name"]
        order.rechnungsadresse_nachname = neworder["billing"]["last_name"]
        order.rechnungsadresse_firma = neworder["billing"]["company"]
        order.rechnungsadresse_adresszeile1 = neworder["billing"]["address_1"]
        order.rechnungsadresse_adresszeile2 = neworder["billing"]["address_2"]
        order.rechnungsadresse_ort = neworder["billing"]["city"]
        order.rechnungsadresse_kanton = neworder["billing"]["state"]
        order.rechnungsadresse_plz = neworder["billing"]["postcode"]
        order.rechnungsadresse_land = neworder["billing"]["country"]
        order.rechnungsadresse_email = neworder["billing"]["email"]
        order.rechnungsadresse_telefon = neworder["billing"]["phone"]

        order.lieferadresse_vorname = neworder["shipping"]["first_name"]
        order.lieferadresse_nachname = neworder["shipping"]["last_name"]
        order.lieferadresse_firma = neworder["shipping"]["company"]
        order.lieferadresse_adresszeile1 = neworder["shipping"]["address_1"]
        order.lieferadresse_adresszeile2 = neworder["shipping"]["address_2"]
        order.lieferadresse_ort = neworder["shipping"]["city"]
        order.lieferadresse_kanton = neworder["shipping"]["state"]
        order.lieferadresse_plz = neworder["shipping"]["postcode"]
        order.lieferadresse_land = neworder["shipping"]["country"]
        order.save()
        log("Bestellung aktualisiert: ", str(order))
        return order

    @classmethod
    def order_import(self):
        with Progress() as progress:
            task_prepare = progress.add_task(
                prefix + " [orange_red1]Bestellungsdownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Bestellung.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            wcapi = self.get_api()
            orderlist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                prefix + " [green]Bestellungen herunterladen...")

            r = wcapi.get("orders?exclude=" + excludeids)
            orderlist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                orderlist += wcapi.get("orders?exclude=" +
                                       excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)
            progress.stop_task(task_download)

            if orderlist:
                task_process = progress.add_task(
                    prefix + " [cyan]Bestellungen verarbeiten...", total=len(orderlist))
                for order in orderlist:
                    self.order_create(order)
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)
        return len(orderlist)

    @classmethod
    def order_bulk_update(self, orders):
        with Progress() as progress:
            task = progress.add_task(
                prefix + " [orange_red1]Bestellungen aktualisieren...", total=orders.count())
            successcount = 0
            errorcount = 0
            for order in orders:
                if order.woocommerceid:
                    self.order_update(order)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)



from django.contrib import admin, messages
from django.utils.html import format_html, mark_safe
from datetime import datetime
from pytz import utc

from kmuhelper.app.models import ToDoNotiz, ToDoVersand, ToDoZahlungseingang, ToDoLagerbestand, ToDoLieferung
from kmuhelper.main.models import Ansprechpartner, Bestellung, Kategorie, Kosten, Kunde, Lieferant, Lieferung, Notiz, Produkt, Zahlungsempfaenger, Einstellung

from kmuhelper.utils import package_version, python_version
from kmuhelper.integrations.woocommerce import WooCommerce

#######

@admin.register(Ansprechpartner)
class AnsprechpartnerAdmin(admin.ModelAdmin):
    fieldsets = [
        ("Name", {'fields': ['name']}),
        ('Daten', {'fields': ['telefon', 'email']})
    ]

    ordering = ('name',)

    list_display = ('name', 'telefon', 'email')
    search_fields = ['name', 'telefon', 'email']


class BestellungInlineBestellungsposten(admin.TabularInline):
    model = Bestellung.produkte.through
    verbose_name = "Bestellungsposten"
    verbose_name_plural = "Bestellungsposten"
    extra = 0

    fieldsets = [
        (None, {'fields': ['produkt', 'bemerkung', 'produktpreis',
                           'menge', 'rabatt', 'mwstsatz', 'zwischensumme']})
    ]

    def get_readonly_fields(self, request, obj=None):
        fields = ["zwischensumme", "mwstsatz", "produkt", "produktpreis"]
        if obj and (obj.versendet or obj.bezahlt):
            fields += ["menge"]
        if obj and obj.bezahlt:
            fields += ["rabatt"]
        return fields

    def has_change_permission(self, request, obj=None):
        return False if (obj and (obj.versendet and obj.bezahlt)) else True

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False if (obj and (obj.versendet or obj.bezahlt)) else True


class BestellungInlineBestellungspostenAdd(admin.TabularInline):
    model = Bestellung.produkte.through
    verbose_name = "Bestellungsposten"
    verbose_name_plural = "Bestellungsposten hinzufügen"
    extra = 0

    autocomplete_fields = ("produkt",)

    fieldsets = [
        (None, {'fields': ['produkt', 'bemerkung', 'menge', 'rabatt']})
    ]

    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request, obj=None):
        return False if (obj and (obj.versendet or obj.bezahlt)) else True

    def has_delete_permission(self, request, obj=None):
        return False

    def has_view_permission(self, request, obj=None):
        return False


class BestellungInlineBestellungskosten(admin.TabularInline):
    model = Bestellung.kosten.through
    verbose_name = "Bestellungskosten"
    verbose_name_plural = "Bestellungskosten"
    extra = 0

    fieldsets = [
        (None, {'fields': ['kosten_name', 'bemerkung',
                           'kostenpreis', 'rabatt', 'mwstsatz', 'zwischensumme']})
    ]

    def get_readonly_fields(self, request, obj=None):
        fields = ["zwischensumme", "mwstsatz", "kostenpreis", "kosten_name"]
        return fields

    def has_change_permission(self, request, obj=None):
        return False if (obj and obj.bezahlt) else True

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False if (obj and obj.bezahlt) else True


class BestellungInlineBestellungskostenAdd(admin.TabularInline):
    model = Bestellung.kosten.through
    verbose_name = "Bestellungskosten"
    verbose_name_plural = "Bestellungskosten hinzufügen"
    extra = 0

    autocomplete_fields = ("kosten",)

    fieldsets = [
        (None, {'fields': ['kosten', 'bemerkung', 'rabatt']})
    ]

    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request, obj=None):
        return False if (obj and obj.bezahlt) else True

    def has_delete_permission(self, request, obj=None):
        return False

    def has_view_permission(self, request, obj=None):
        return False


@admin.register(Bestellung)
class BestellungsAdmin(admin.ModelAdmin):
    list_display = ('id', 'datum', 'kunde', 'status', 'zahlungsmethode',
                    'versendet', 'bezahlt', 'fix_summe', 'html_notiz')
    list_filter = ('status', 'bezahlt', 'versendet', 'zahlungsmethode')
    search_fields = ['id', 'datum', 'notiz__name', 'notiz__beschrieb', 'kundennotiz', 'trackingnummer', 'rechnungsadresse_vorname',
                     'rechnungsadresse_nachname', 'rechnungsadresse_firma', 'rechnungsadresse_plz', 'rechnungsadresse_ort']

    ordering = ("versendet", "bezahlt", "-datum")

    inlines = [BestellungInlineBestellungsposten, BestellungInlineBestellungspostenAdd,
               BestellungInlineBestellungskosten, BestellungInlineBestellungskostenAdd]

    autocomplete_fields = ("kunde",)

    save_on_top = True

    list_select_related = ["kunde", "notiz"]

    def get_fieldsets(self, request, obj=None):
        if obj:
            return [
                ('Einstellungen', {'fields': [
                 'zahlungsempfaenger', 'ansprechpartner']}),
                ('Infos', {'fields': ['name', 'datum', 'status']}),
                ('Lieferung', {'fields': ['versendet', 'trackingnummer']}),
                ('Bezahlung', {'fields': [
                 'bezahlt', 'zahlungsmethode', 'summe', 'summe_mwst', 'summe_gesamt']}),
                ('Kunde', {'fields': ['kunde']}),
                ('Notizen', {'fields': ['kundennotiz', 'html_notiz'], 'classes': [
                 "collapse start-open"]}),
                ('Rechnungsadresse', {'fields': ['rechnungsadresse_vorname', 'rechnungsadresse_nachname', 'rechnungsadresse_firma', 'rechnungsadresse_adresszeile1', 'rechnungsadresse_adresszeile2',
                                                 'rechnungsadresse_plz', 'rechnungsadresse_ort', 'rechnungsadresse_kanton', 'rechnungsadresse_land', 'rechnungsadresse_email', 'rechnungsadresse_telefon'], 'classes': ["collapse default-open"]}),
                ('Lieferadresse', {'fields': ['lieferadresse_vorname', 'lieferadresse_nachname', 'lieferadresse_firma', 'lieferadresse_adresszeile1',
                                              'lieferadresse_adresszeile2', 'lieferadresse_plz', 'lieferadresse_ort', 'lieferadresse_kanton', 'lieferadresse_land'], 'classes': ["collapse start-open"]})
            ]
        else:
            return [
                ('Einstellungen', {'fields': [
                 'zahlungsempfaenger', 'ansprechpartner']}),
                ('Lieferung', {'fields': ['trackingnummer'], 'classes': [
                 "collapse"]}),
                ('Bezahlung', {'fields': ['zahlungsmethode'], 'classes': [
                 "collapse start-open"]}),
                ('Kunde', {'fields': ['kunde']}),
                ('Notizen', {'fields': ['kundennotiz'],
                             'classes': ["collapse start-open"]}),
            ]

    def get_readonly_fields(self, request, obj=None):
        rechnungsadresse = ['rechnungsadresse_vorname', 'rechnungsadresse_nachname', 'rechnungsadresse_firma', 'rechnungsadresse_adresszeile1', 'rechnungsadresse_adresszeile2',
                            'rechnungsadresse_plz', 'rechnungsadresse_ort', 'rechnungsadresse_kanton', 'rechnungsadresse_land', 'rechnungsadresse_email', 'rechnungsadresse_telefon']
        lieferadresse = ['lieferadresse_vorname', 'lieferadresse_nachname', 'lieferadresse_firma', 'lieferadresse_adresszeile1',
                         'lieferadresse_adresszeile2', 'lieferadresse_plz', 'lieferadresse_ort', 'lieferadresse_kanton', 'lieferadresse_land']
        fields = ['html_notiz', 'name', 'trackinglink',
                  'summe', 'summe_mwst', 'summe_gesamt']
        if obj:
            if obj.versendet:
                fields += ['versendet', 'trackingnummer'] + lieferadresse
            if obj.bezahlt:
                fields += ['bezahlt', 'zahlungsmethode'] + rechnungsadresse
            if obj.woocommerceid:
                fields += ["kundennotiz"]
        return fields

    def als_bezahlt_markieren(self, request, queryset):
        successcount = 0
        errorcount = 0
        for bestellung in queryset.all():
            if bestellung.bezahlt:
                errorcount += 1
            else:
                bestellung.bezahlt = True
                if bestellung.versendet:
                    bestellung.status = "completed"
                bestellung.save()
                successcount += 1
        messages.success(request, (('{} Bestellungen' if successcount !=
                                    1 else 'Eine Bestellung') + ' als bezahlt markiert!').format(successcount))
        if errorcount:
            messages.error(request, ('Beim als bezahlt markieren von ' + ('{} Bestellungen' if errorcount !=
                                                                          1 else 'einer Bestellung') + ' ist ein Fehler aufgetreten!').format(errorcount))
    als_bezahlt_markieren.short_description = "Als bezahlt markieren"

    def wc_update(self, request, queryset):
        result = WooCommerce.order_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Bestellungen' if result[0] != 1 else 'Eine Bestellung') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Bestellungen' if result[1] != 1 else 'einer Bestellung') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))
    wc_update.short_description = "Bestellungen von WooCommerce aktualisieren"

    actions = [als_bezahlt_markieren, wc_update]

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)

        # stock warnings
        if obj:
            stock = obj.get_future_stock()
            for p_id in stock:
                s = stock[p_id]
                p = s["product"]

                p_name = p["name"]
                p_artikelnummer = p["artikelnummer"]
                p_adminurl = p["adminurl"]

                n_current = s["current"]
                n_going = s["going"]
                n_coming = s["coming"]
                n_min = s["min"]

                adminlink = mark_safe(f'<a href="{p_adminurl}">{p_name}</a>')
                stockdata = f"Aktuell: { n_current } | Ausgehend: { n_going }" + (
                        f" | Eingehend: { n_coming }" if n_coming else "")

                formatdata = (mark_safe(adminlink), p_artikelnummer, stockdata)

                if n_current-n_going < 0:
                    msg = format_html('Zu wenig Lagerbestand bei "{}" [{}]: {}', *formatdata)
                    messages.error(request, msg)
                elif n_current-n_going < n_min:
                    msg = format_html('Knapper Lagerbestand bei "{}" [{}]: {}', *formatdata)
                    messages.warning(request, msg)


class KategorienInlineUntergeordneteKategorien(admin.TabularInline):
    model = Produkt.kategorien.through
    verbose_name = "Produkt in dieser Kategorie"
    verbose_name_plural = "Produkte in dieser Kategorie"
    extra = 0


@admin.register(Kategorie)
class KategorienAdmin(admin.ModelAdmin):
    fieldsets = [
        ('Infos', {'fields': ['name', 'beschrieb', 'bildlink']}),
        ('Übergeordnete Kategorie', {'fields': ['uebergeordnete_kategorie']})
    ]

    list_display = ('clean_name', 'clean_beschrieb',
                    'uebergeordnete_kategorie', 'bild', 'anzahl_produkte')
    search_fields = ['name', 'beschrieb']

    ordering = ("uebergeordnete_kategorie", "name")

    inlines = [KategorienInlineUntergeordneteKategorien]

    list_select_related = ("uebergeordnete_kategorie",)

    def wc_update(self, request, queryset):
        result = WooCommerce.category_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Kategorien' if result[0] != 1 else 'Eine Kategorie') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Kategorien' if result[1] != 1 else 'einer Kategorie') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))
    wc_update.short_description = "Kategorien von WooCommerce aktualisieren"

    actions = ["wc_update"]


@admin.register(Kosten)
class KostenAdmin(admin.ModelAdmin):
    list_display = ["clean_name", "preis", "mwstsatz"]

    search_fields = ('name', 'preis')

    fieldsets = [
        (None, {"fields": ("name", "preis", "mwstsatz")})
    ]


@admin.register(Kunde)
class KundenAdmin(admin.ModelAdmin):
    def get_fieldsets(self, request, obj=None):
        if obj:
            return [
                ('Infos', {'fields': ['vorname', 'nachname',
                                      'firma', 'email', 'sprache']}),
                ('Rechnungsadresse', {'fields': [('rechnungsadresse_vorname', 'rechnungsadresse_nachname'), 'rechnungsadresse_firma', ('rechnungsadresse_adresszeile1', 'rechnungsadresse_adresszeile2'), (
                    'rechnungsadresse_plz', 'rechnungsadresse_ort'), ('rechnungsadresse_kanton', 'rechnungsadresse_land'), ('rechnungsadresse_email', 'rechnungsadresse_telefon')]}),
                ('Lieferadresse', {'fields': [('lieferadresse_vorname', 'lieferadresse_nachname'), 'lieferadresse_firma', ('lieferadresse_adresszeile1', 'lieferadresse_adresszeile2'), (
                    'lieferadresse_plz', 'lieferadresse_ort'), ('lieferadresse_kanton', 'lieferadresse_land')], 'classes': ["collapse start-open"]}),
                ('Diverses', {'fields': [
                 'webseite', 'bemerkung', 'html_notiz']}),
                ('Erweitert', {'fields': [
                 'zusammenfuegen'], 'classes': ["collapse"]})
            ]
        else:
            return [
                ('Infos', {'fields': ['vorname', 'nachname',
                                      'firma', 'email', 'sprache']}),
                ('Rechnungsadresse', {'fields': [('rechnungsadresse_vorname', 'rechnungsadresse_nachname'), 'rechnungsadresse_firma', ('rechnungsadresse_adresszeile1', 'rechnungsadresse_adresszeile2'), (
                    'rechnungsadresse_plz', 'rechnungsadresse_ort'), ('rechnungsadresse_kanton', 'rechnungsadresse_land'), ('rechnungsadresse_email', 'rechnungsadresse_telefon')]}),
                ('Lieferadresse', {'fields': [('lieferadresse_vorname', 'lieferadresse_nachname'), 'lieferadresse_firma', ('lieferadresse_adresszeile1', 'lieferadresse_adresszeile2'), (
                    'lieferadresse_plz', 'lieferadresse_ort'), ('lieferadresse_kanton', 'lieferadresse_land')], 'classes': ["collapse start-open"]}),
                ('Diverses', {'fields': ['webseite', 'bemerkung']})
            ]

    ordering = ('rechnungsadresse_plz', 'firma', 'nachname', 'vorname')

    list_display = ('id', 'firma', 'nachname', 'vorname', 'rechnungsadresse_plz',
                    'rechnungsadresse_ort', 'email', 'avatar', 'html_notiz')
    search_fields = ['id', 'nachname', 'vorname', 'firma', 'email', 'benutzername', 'rechnungsadresse_vorname', 'rechnungsadresse_nachname', 'rechnungsadresse_firma', 'rechnungsadresse_adresszeile1', 'rechnungsadresse_adresszeile2', 'rechnungsadresse_ort', 'rechnungsadresse_kanton', 'rechnungsadresse_plz', 'rechnungsadresse_land',
                     'rechnungsadresse_email', 'rechnungsadresse_telefon', 'lieferadresse_vorname', 'lieferadresse_nachname', 'lieferadresse_firma', 'lieferadresse_adresszeile1', 'lieferadresse_adresszeile2', 'lieferadresse_ort', 'lieferadresse_kanton', 'lieferadresse_kanton', 'lieferadresse_plz', 'lieferadresse_land', 'webseite', 'notiz__name', 'notiz__beschrieb']

    readonly_fields = ["html_notiz"]

    actions = ["wc_update"]

    list_select_related = ["notiz"]

    save_on_top = True

    def wc_update(self, request, queryset):
        result = WooCommerce.customer_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Kunden' if result[0] != 1 else 'Ein Kunde') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Kunden' if result[1] != 1 else 'einem Kunden') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))
    wc_update.short_description = "Kunden von WooCommerce aktualisieren"


@admin.register(Lieferant)
class LieferantenAdmin(admin.ModelAdmin):
    fieldsets = [
        ('Infos', {'fields': ['kuerzel', 'name']}),
        ('Firma', {'fields': ['webseite', 'telefon', 'email']}),
        ('Texte', {'fields': ['adresse', 'notiz'], 'classes': ["collapse"]}),
        ('Ansprechpartner', {'fields': [
         'ansprechpartner', 'ansprechpartnertel', 'ansprechpartnermail'], 'classes': ["collapse"]})
    ]

    ordering = ('kuerzel',)

    list_display = ('kuerzel', 'name', 'notiz')
    search_fields = ['kuerzel', 'name', 'adresse', 'notiz']


class LieferungInlineProdukte(admin.TabularInline):
    model = Lieferung.produkte.through
    verbose_name = "Produkt"
    verbose_name_plural = "Produkte"
    extra = 0

    readonly_fields = ("produkt",)

    fields = ("produkt", "menge",)

    def has_change_permission(self, request, obj=None):
        return not (obj and obj.eingelagert)

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return not (obj and obj.eingelagert)


class LieferungInlineProdukteAdd(admin.TabularInline):
    model = Lieferung.produkte.through
    verbose_name = "Produkt"
    verbose_name_plural = "Produkte hinzufügen"
    extra = 0

    autocomplete_fields = ("produkt",)

    fields = ("produkt", "menge",)

    def has_view_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request, obj=None):
        return not (obj and obj.eingelagert)

    def has_delete_permission(self, request, obj=None):
        return False


@admin.register(Lieferung)
class LieferungenAdmin(admin.ModelAdmin):
    list_display = ('name', 'datum', 'anzahlprodukte',
                    'lieferant', 'eingelagert', 'html_notiz')
    list_filter = ("eingelagert", "lieferant", )

    search_fields = ["name", "datum", "lieferant", "notiz__name", "notiz__beschrieb"]

    readonly_fields = ["html_notiz"]

    ordering = ('-datum',)

    fieldsets = [
        ('Infos', {'fields': ['name', 'html_notiz']}),
        ('Lieferant', {'fields': ['lieferant']})
    ]

    inlines = [LieferungInlineProdukte, LieferungInlineProdukteAdd]

    save_on_top = True

    actions = ["einlagern"]

    def einlagern(self, request, queryset):
        successcount = 0
        errorcount = 0
        for lieferung in queryset.all():
            if lieferung.einlagern():
                successcount += 1
            else:
                errorcount += 1
        messages.success(request, (('{} Lieferungen' if successcount !=
                                    1 else 'Eine Lieferung') + ' wurde eingelagert!').format(successcount))
        if errorcount:
            messages.error(request, ('Beim Einlagern von ' + ('{} Lieferungen' if errorcount !=
                                                              1 else 'einer Lieferung') + ' ist ein Fehler aufgetreten!').format(errorcount))
    einlagern.short_description = "Lieferungen einlagern"


@admin.register(Notiz)
class NotizenAdmin(admin.ModelAdmin):
    list_display = ["name", "beschrieb", "priority", "erledigt", "erstellt_am"]
    list_filter = ["erledigt", "priority"]

    readonly_fields = ["links"]

    ordering = ["erledigt", "-priority", "erstellt_am"]

    search_fields = ("name", "beschrieb")

    def get_fieldsets(self, request, obj=None):
        if obj:
            return [
                ("Infos", {"fields": ["name", "beschrieb"]}),
                ("Daten", {"fields": ["erledigt", "priority"]}),
                ("Verknüpfungen", {"fields": ["links"]}),
            ]
        else:
            return [
                ("Infos", {"fields": ["name", "beschrieb"]}),
                ("Daten", {"fields": ["erledigt", "priority"]}),
            ]

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        if obj is None:
            form.base_fields['beschrieb'].initial = ""
            if "from_bestellung" in request.GET:
                form.base_fields['name'].initial = 'Bestellung #' + \
                    request.GET.get("from_bestellung")
                form.base_fields['beschrieb'].initial += '\n\n[Bestellung #' + \
                    request.GET.get("from_bestellung") + "]"
            if "from_produkt" in request.GET:
                form.base_fields['name'].initial = 'Produkt #' + \
                    request.GET.get("from_produkt")
                form.base_fields['beschrieb'].initial += '\n\n[Produkt #' + \
                    request.GET.get("from_produkt") + "]"
            if "from_kunde" in request.GET:
                form.base_fields['name'].initial = 'Kunde #' + \
                    request.GET.get("from_kunde")
                form.base_fields['beschrieb'].initial += '\n\n[Kunde #' + \
                    request.GET.get("from_kunde") + "]"
            if "from_lieferung" in request.GET:
                form.base_fields['name'].initial = 'Lieferung #' + \
                    request.GET.get("from_lieferung")
                form.base_fields['beschrieb'].initial += '\n\n[Lieferung #' + \
                    request.GET.get("from_lieferung") + "]"
        return form

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if not change:
            if "from_bestellung" in request.GET:
                if Bestellung.objects.filter(pk=request.GET["from_bestellung"]).exists():
                    bestellung = Bestellung.objects.get(
                        pk=request.GET["from_bestellung"])
                    obj.bestellung = bestellung
                    obj.save()
                    messages.info(
                        request, "Bestellung #" + str(bestellung.pk) + " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Bestellung #" + request.GET["from_bestellung"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")
            if "from_produkt" in request.GET:
                if Produkt.objects.filter(pk=request.GET["from_produkt"]).exists():
                    produkt = Produkt.objects.get(
                        pk=request.GET["from_produkt"])
                    obj.produkt = produkt
                    obj.save()
                    messages.info(
                        request, "Produkt #" + str(produkt.pk) + " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Produkt #" + request.GET["from_produkt"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")
            if "from_kunde" in request.GET:
                if Kunde.objects.filter(pk=request.GET["from_kunde"]).exists():
                    kunde = Kunde.objects.get(pk=request.GET["from_kunde"])
                    obj.kunde = kunde
                    obj.save()
                    messages.info(request, "Kunde #" + str(kunde.pk) +
                                  " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Kunde #" + request.GET["from_kunde"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")
            if "from_lieferung" in request.GET:
                if Lieferung.objects.filter(pk=request.GET["from_lieferung"]).exists():
                    lieferung = Lieferung.objects.get(
                        pk=request.GET["from_lieferung"])
                    obj.lieferung = lieferung
                    obj.save()
                    messages.info(request, "Lieferung #" + str(lieferung.pk) +
                                  " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Lieferung #" + request.GET["from_lieferung"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")


class ProduktInlineProduktkategorien(admin.TabularInline):
    model = Produkt.kategorien.through
    extra = 0


@admin.register(Produkt)
class ProduktAdmin(admin.ModelAdmin):
    def get_fieldsets(self, request, obj=None):
        if obj:
            return [
                ('Infos', {'fields': ['artikelnummer', 'name']}),
                ('Beschrieb', {'fields': [
                 'kurzbeschrieb', 'beschrieb'], 'classes': ["collapse start-open"]}),
                ('Daten', {'fields': [
                 'mengenbezeichnung', 'verkaufspreis', 'mwstsatz', 'lagerbestand', 'soll_lagerbestand']}),
                ('Lieferant', {'fields': [
                 'lieferant', 'lieferant_preis', 'lieferant_artikelnummer', 'lieferant_url'], 'classes': [
                    "collapse start-open"]}),
                ('Aktion', {'fields': ['aktion_von', 'aktion_bis', 'aktion_preis'], 'classes': [
                 "collapse start-open"]}),
                ('Links', {'fields': ['datenblattlink', 'bildlink'], 'classes': [
                 "collapse"]}),
                # packlistenbemerkung
                ('Bemerkung / Notiz',
                 {'fields': ['bemerkung', 'html_notiz'], 'classes': ["collapse start-open"]})
            ]
        else:
            return [
                ('Infos', {'fields': ['artikelnummer', 'name']}),
                ('Beschrieb', {'fields': [
                 'kurzbeschrieb', 'beschrieb'], 'classes': ["collapse start-open"]}),
                ('Daten', {'fields': [
                 'mengenbezeichnung', 'verkaufspreis', 'mwstsatz', 'lagerbestand', 'soll_lagerbestand']}),
                ('Lieferant', {'fields': [
                 'lieferant', 'lieferant_preis', 'lieferant_artikelnummer', 'lieferant_url'], 'classes': [
                    "collapse start-open"]}),
                ('Aktion', {'fields': ['aktion_von', 'aktion_bis', 'aktion_preis'], 'classes': [
                 "collapse start-open"]}),
                ('Links', {'fields': ['datenblattlink', 'bildlink'], 'classes': [
                 "collapse"]}),
                # packlistenbemerkung
                ('Bemerkung', {'fields': ['bemerkung'], 'classes': [
                 "collapse start-open"]})
            ]

    ordering = ('artikelnummer', 'name')

    list_display = ('artikelnummer', 'clean_name', 'clean_kurzbeschrieb',
                    'clean_beschrieb', 'preis', 'in_aktion', 'lagerbestand', 'bild', 'html_notiz')
    list_filter = ('lieferant', 'kategorien', 'lagerbestand')
    search_fields = ['artikelnummer', 'name', 'kurzbeschrieb',
                     'beschrieb', 'bemerkung', 'notiz__name', 'notiz__beschrieb']

    readonly_fields = ["html_notiz"]

    inlines = (ProduktInlineProduktkategorien,)

    save_on_top = True

    actions = ["wc_update", "lagerbestand_zuruecksetzen", "aktion_beenden"]

    list_select_related = ["notiz"]

    def wc_update(self, request, queryset):
        result = WooCommerce.product_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Produkte' if result[0] != 1 else 'Ein Produkt') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Produkten' if result[1] != 1 else 'einem Produkt') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))
    wc_update.short_description = "Produkte von WooCommerce aktualisieren"

    def lagerbestand_zuruecksetzen(self, request, queryset):
        for produkt in queryset.all():
            produkt.lagerbestand = 0
            produkt.save()
        messages.success(request, (('Lagerbestand von {} Produkten' if queryset.count(
        ) != 1 else 'Lagerbestand von einem Produkt') + ' zurückgesetzt!').format(queryset.count()))
    lagerbestand_zuruecksetzen.short_description = "Lagerbestand zurücksetzen"

    def aktion_beenden(self, request, queryset):
        for produkt in queryset.all():
            produkt.aktion_bis = datetime.now(utc)
            produkt.save()
        messages.success(request, (('Aktion von {} Produkten' if queryset.count(
        ) != 1 else 'Aktion von einem Produkt') + ' beendet!').format(queryset.count()))
    aktion_beenden.short_description = "Aktion beenden"

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if obj:
            n_current = obj.lagerbestand
            n_going = obj.get_reserved_stock()
            n_coming = obj.get_incoming_stock()
            n_min = obj.soll_lagerbestand

            stockdata = f"Aktuell: { n_current } | Offene Bestellungen: { n_going }" + (
                f" | Kommende Lieferungen: { n_coming }" if n_coming else "")

            if n_current-n_going < 0:
                messages.error(request, f"Zu wenig Lagerbestand! "+stockdata)
            elif n_current-n_going < n_min:
                messages.warning(request, f"Knapper Lagerbestand! "+stockdata)


@admin.register(Zahlungsempfaenger)
class ZahlungsempfaengerAdmin(admin.ModelAdmin):
    fieldsets = [
        ("Infos", {"fields": [
         "qriban", "logourl", "firmenname", "firmenuid"]}),
        ("Adresse", {"fields": ["adresszeile1", "adresszeile2", "land"]}),
        ("Daten", {"fields": ["email", "telefon", "webseite"]})
    ]

    list_display = ('firmenname', 'firmenuid', 'qriban')
    list_filter = ('land',)
    search_fields = ['firmenname', 'adresszeile1',
                     'adresszeile2', 'qriban', 'firmenuid']

    save_on_top = True

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if obj and not obj.has_valid_qr_iban():
            messages.error(request, "Ungültige QR-IBAN!")
        if obj and not obj.has_valid_uid():
            messages.warning(request, "Ungültige UID!")


################### Einstellungen

@admin.register(Einstellung)
class EinstellungenAdmin(admin.ModelAdmin):
    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    list_display = ('name', 'get_inhalt')
    ordering = ('name',)

    search_fields = ['name', 'char', 'text', 'inte', 'floa', 'url', 'email']

    readonly_fields = ["id", "name"]

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            ('Name', {'fields': ['name']}),
            ('Inhalt', {'fields': [
                'char' if obj.typ == 'char' else
                'text' if obj.typ == 'text' else
                'boo' if obj.typ == 'bool' else
                'inte' if obj.typ == 'int' else
                'floa' if obj.typ == 'float' else
                'url' if obj.typ == 'url' else
                'email' if obj.typ == 'email' else ''
            ]}),
        ]
        return fieldsets


#

modeladmins = [
    (Ansprechpartner, AnsprechpartnerAdmin),
    (Bestellung, BestellungsAdmin),
    (Kategorie, KategorienAdmin),
    (Kosten, KostenAdmin),
    (Kunde, KundenAdmin),
    (Lieferant, LieferantenAdmin),
    (Lieferung, LieferungenAdmin),
    (Notiz, NotizenAdmin),
    (Produkt, ProduktAdmin),
    (Zahlungsempfaenger, ZahlungsempfaengerAdmin),
    (Einstellung, EinstellungenAdmin),
]



from django.core.management.base import BaseCommand, CommandError
from kmuhelper.models import Kunde
from kmuhelper.utils import getfirstindex

import csv

class Command(BaseCommand):
    help = 'Imports customers from a .csv file.'

    def add_arguments(self, parser):
        parser.add_argument('filepath', help="Absolute filepath to the .csv file", type=str)

    def handle(self, *args, **options):
        self.stdout.write(self.style.SUCCESS("Import gestartet!"))
        try:
            with open(options["filepath"], newline='', encoding="utf-8-sig") as f:
                reader = csv.reader(f)
                rows = list(reader)

            indexrow = rows.pop(0)

            ###
            index_id = getfirstindex(indexrow, ["Nummer", "ID"])

            index_email = getfirstindex(indexrow, ["Email", "E-Mail","EMail","EMAIL"])
            index_vorname = getfirstindex(indexrow, ["Vorname"])
            index_nachname = getfirstindex(indexrow, ["Name", "Nachname"])
            index_sprache = getfirstindex(indexrow, ["Sprache", "SPRACHE"])

            index_firma = getfirstindex(indexrow, ["Firma"])
            index_strasse = getfirstindex(indexrow, ["Strasse"])
            index_ort = getfirstindex(indexrow, ["Ort"])
            index_plz = getfirstindex(indexrow, ["PLZ","Plz","Postleitzahl"])
            index_land = getfirstindex(indexrow, ["Land"])
            index_telefon = getfirstindex(indexrow, ["Telefon", "Telefon1"])

            index_adresszeile2 = getfirstindex(indexrow, ["Adresszeile2", "Zusatz"])

            index_notiz = getfirstindex(indexrow, ["Notiz"])
            index_web = getfirstindex(indexrow, ["Webseite", "Homepage"])
            ###
            for row in rows:
                id = row[index_id] if index_id is not None else None

                email = row[index_email] if index_email is not None else ""
                vorname = row[index_vorname] if index_vorname is not None else ""
                nachname = row[index_nachname] if index_nachname is not None else ""
                sprache =   ("de" if row[index_sprache].upper() in ["D","DE","DEUTSCH"] else
                            ("fr" if row[index_sprache].upper() in ["F","FR","FRANZÖSISCH"] else
                            ("it" if row[index_sprache].upper() in ["I","IT","ITALIENISCH"] else
                            ("en" if row[index_sprache].upper() in ["E","EN","ENGLISCH"] else
                            "de")))) if index_sprache is not None else "de"

                firma = row[index_firma] if index_firma is not None else ""
                adresszeile1 = row[index_strasse] if index_strasse is not None else ""
                adresszeile2 = row[index_adresszeile2] if index_adresszeile2 is not None else ""
                ort = row[index_ort] if index_ort is not None else ""
                plz = row[index_plz] if index_plz is not None else ""
                land = row[index_land] if index_land is not None else ""
                telefon = row[index_telefon] if index_telefon is not None else ""

                notiz = row[index_notiz] if index_notiz is not None else ""
                webseite = row[index_web] if index_web is not None else ""

                oldwithid = Kunde.objects.filter(id=id).exists()
                oldwithemail = Kunde.objects.filter(email=email).exists() if email else False
                oldwithidandemail = Kunde.objects.filter(id=id, email=email).exists()

                self.stdout.write(self.style.SUCCESS(""))

                if not oldwithid and not oldwithemail:
                    self.stdout.write(self.style.SUCCESS("Kunden erstellen:"+
                        "\n  ID:         "+ str(id)+
                        "\n  E-Mail:     "+ email+
                        "\n  Vorname:    "+ vorname+
                        "\n  Nachname:   "+ nachname+
                        "\n  Firma:      "+ firma
                    ))
                    Kunde.objects.create(
                        id=id,

                        email=email,
                        vorname=vorname,
                        nachname=nachname,
                        firma=firma,
                        sprache=sprache,

                        rechnungsadresse_vorname = vorname,
                        rechnungsadresse_nachname = nachname,
                        rechnungsadresse_firma = firma,
                        rechnungsadresse_adresszeile1 = adresszeile1,
                        rechnungsadresse_adresszeile2 = adresszeile2,
                        rechnungsadresse_ort = ort,
                        #rechnungsadresse_kanton = "",
                        rechnungsadresse_plz = plz,
                        rechnungsadresse_land = land,
                        rechnungsadresse_email = email,
                        rechnungsadresse_telefon = telefon,

                        lieferadresse_vorname = vorname,
                        lieferadresse_nachname = nachname,
                        lieferadresse_firma = firma,
                        lieferadresse_adresszeile1 = adresszeile1,
                        lieferadresse_adresszeile2 = adresszeile2,
                        lieferadresse_ort = ort,
                        #lieferadresse_kanton = "",
                        lieferadresse_plz = plz,
                        lieferadresse_land = land,

                        webseite = webseite,
                        notiz = notiz
                    )
                elif oldwithidandemail:
                    self.stdout.write(self.style.SUCCESS("Kunde existiert bereits!"+
                        "\n  ID:         "+ str(id)+
                        "\n  E-Mail:     "+ email+
                        "\n  Vorname:    "+ vorname+
                        "\n  Nachname:   "+ nachname+
                        "\n  Firma:      "+ firma
                    ))
                elif oldwithemail and not oldwithid:
                    old = Kunde.objects.get(email=email)
                    self.stdout.write(self.style.SUCCESS("Kunde mit der E-Mail '"+email+"' existiert bereits, jedoch ist die ID unterschiedlich!"+
                        "\n  KMUHelper-Daten:"+
                        "\n    ID:       "+ str(old.id)+
                        "\n    Vorname:  "+ old.vorname+
                        "\n    Nachname: "+ old.nachname+
                        "\n    Firma:    "+ old.rechnungsadresse_firma+
                        "\n  CSV Daten:"+
                        "\n    ID:       "+ str(id)+
                        "\n    Vorname:  "+ vorname+
                        "\n    Nachname: "+ nachname+
                        "\n    Firma:    "+ firma
                    ))
                    if input("ID aus der CSV-Datei verwenden? [y/n] ").lower() in ["y","yes","j","ja","aktualisieren"]:
                        oldid = old.id
                        old.id = int(id)
                        old.save()
                        new = old
                        old = Kunde.objects.get(id=oldid)
                        new.zusammenfuegen = old
                        new.save()
                        self.stdout.write(self.style.SUCCESS("ID aktualisiert!"))
                elif not oldwithemail and oldwithid:
                    old = Kunde.objects.get(id=id)
                    self.stdout.write(self.style.SUCCESS("Kunde mit der ID '"+str(id)+"' existiert bereits, jedoch ist die E-Mail unterschiedlich!"+
                        "\n  KMUHelper-Daten:"+
                        "\n    E-Mail:   "+ old.email+
                        "\n    Vorname:  "+ old.vorname+
                        "\n    Nachname: "+ old.nachname+
                        "\n    Firma:    "+ old.rechnungsadresse_firma+
                        "\n  CSV Daten:"+
                        "\n    E-Mail:   "+ email+
                        "\n    Vorname:  "+ vorname+
                        "\n    Nachname: "+ nachname+
                        "\n    Firma:    "+ firma
                    ))
                    if input("E-Mail aus der CSV-Datei verwenden? [y/n] ").lower() in ["y","yes","j","ja","aktualisieren"]:
                        old.email = email
                        old.save()
                        self.stdout.write(self.style.SUCCESS("E-Mail aktualisiert!"))
                elif oldwithemail and oldwithid:
                    self.stdout.write(self.style.SUCCESS("Irgendetwas stimmt nicht! Es besteht ein Kunde mit der E-Mail '"+email+"' aber ein Anderer mit der ID '"+str(id)+"'."))

        except FileNotFoundError:
            raise CommandError('Datei nicht gefunden!')



from rich import print

from kmuhelper.main.models import Einstellung, Geheime_Einstellung

def log(string, *args):
    print("[deep_pink4][KMUHelper][/] -", string, *args)

# Einstellungen festlegen

log("startup.py running...")

try:
    Einstellung.objects.get_or_create(id="wc-url", typ="url", name="WooCommerce Shop-Url")
    Einstellung.objects.get_or_create(id="email-stock-warning-receiver", typ="email", name="E-Mail für Warnungen zum Lagerbestand")
    Einstellung.objects.get_or_create(id="email-show-buttons", typ="bool", name="E-Mail Knöpfe anzeigen")

    Geheime_Einstellung.objects.get_or_create(id="wc-consumer_key")
    Geheime_Einstellung.objects.get_or_create(id="wc-consumer_secret")
    Geheime_Einstellung.objects.get_or_create(id="wc-url")
except Exception as e:
    log("startup.py failed: Error while adding settings:", e)

log("startup.py ended")


from django import template
from kmuhelper.main.models import Geheime_Einstellung, Einstellung

register = template.Library()

##########

@register.simple_tag(takes_context=True, name="kmuhelper_woocommerce_connected")
def kmuhelper_woocommerce_connected(context):
    is_connected = bool(
        Geheime_Einstellung.objects.filter(id="wc-url").exists() and
        Geheime_Einstellung.objects.get(id="wc-url").inhalt
    )
    return is_connected


@register.simple_tag(takes_context=True, name="kmuhelper_email_show_buttons")
def kmuhelper_email_show_buttons(context):
    show = bool(
        Einstellung.objects.filter(id="email-show-buttons").exists() and
        Einstellung.objects.get(id="email-show-buttons").inhalt
    )
    return show


@register.filter
def kmuhelper_replace_start_url(value):
    value = value.replace('<a href="/admin/">Start</a>', '')
    value = value.replace('&rsaquo; <a href="/admin/kmuhelper/">KMUHelper</a>','<a href="/admin/kmuhelper/">KMUHelper Admin</a>')
    return value



from django.urls import path, include
from django.contrib.auth import views as auth_views
from django.conf import settings

from kmuhelper.main import views

#######################

urlpatterns = [
    path('kunde/<object_id>/email-registriert', views.kunde_email_registriert, name='kunde-email-registriert'),
    path('lieferant/<object_id>/zuordnen', views.lieferant_zuordnen, name='lieferant-zuordnen'),
    path('lieferung/<object_id>/einlagern', views.lieferung_einlagern, name='lieferung-einlagern'),
    path('bestellung/<object_id>/pdf', views.bestellung_pdf_ansehen, name='bestellung-pdf-ansehen'),
    path('bestellung/<object_id>/pdf/ankundensenden', views.bestellung_pdf_an_kunden_senden, name='bestellung-pdf-an-kunden-senden'),

    path('public/order/<order_id>/<order_key>/', views.public_view_order, name='public-view-order'),
]





from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseBadRequest
from django.shortcuts import render, redirect
from django.urls import reverse, reverse_lazy
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt

from kmuhelper.main.models import Einstellung, Geheime_Einstellung, Produkt, Kunde, Kategorie, Lieferant, Lieferung, Bestellung, Bestellungsposten

###############

# Create your views here.


@login_required(login_url=reverse_lazy("admin:login"))
def lieferant_zuordnen(request, object_id):
    if request.method == "POST":
        if Lieferant.objects.filter(id=int(object_id)).exists():
            count = Lieferant.objects.get(id=int(object_id)).zuordnen()
            messages.success(request, ('Lieferant wurde ' + (('{} neuen Produkten' if count > 1 else 'einem neuen Produkt')
                                                            if count != 0 else "keinem neuen Produkt") + ' zugeordnet!').format(count))
        else:
            messages.error(request, "Lieferant konnte nicht gefunden werden!")
        return redirect(reverse("admin:kmuhelper_lieferant_change", args=[object_id]))
    else:
        return render(request, "admin/kmuhelper/_confirm.html", {"action": "Lieferant zuordnen"})

@login_required(login_url=reverse_lazy("admin:login"))
def lieferung_einlagern(request, object_id):
    if request.method == "POST":
        if Lieferung.objects.filter(id=int(object_id)).exists():
            if Lieferung.objects.get(id=int(object_id)).einlagern():
                messages.success(request, "Lieferung eingelagert!")
            else:
                messages.error(
                    request, "Lieferung konnte nicht eingelagert werden!")
        else:
            messages.error(request, "Lieferung konnte nicht gefunden werden!")
        return redirect(reverse("admin:kmuhelper_lieferung_change", args=[object_id]))
    else:
        return render(request, "admin/kmuhelper/_confirm.html", {"action": "Lieferung einlagern"})


@login_required(login_url=reverse_lazy("admin:login"))
def kunde_email_registriert(request, object_id):
    if request.method == "POST":
        if Kunde.objects.filter(id=int(object_id)).exists():
            if Kunde.objects.get(id=int(object_id)).send_register_mail():
                messages.success(request, "Registrierungsmail gesendet!")
            else:
                messages.error(
                    request, "Registrierungsmail konnte nicht gesendet werden!")
        else:
            messages.error(request, "Kunde konnte nicht gefunden werden!")
        return redirect(reverse("admin:kmuhelper_kunde_change", args=[object_id]))
    else:
        return render(request, "admin/kmuhelper/_confirm.html", {"action": "Registrierungsmail versenden"})


@login_required(login_url=reverse_lazy("admin:login"))
def bestellung_pdf_ansehen(request, object_id):
    if Bestellung.objects.filter(id=int(object_id)).exists():
        obj = Bestellung.objects.get(id=int(object_id))
        lieferschein = bool("lieferschein" in dict(request.GET))
        digital = not bool("druck" in dict(request.GET))
        pdf = obj.get_pdf(lieferschein=lieferschein, digital=digital)
        return pdf
    else:
        messages.error(request, "Bestellung wurde nicht gefunden!")
        return redirect(reverse("admin:kmuhelper_bestellung_changelist"))


@login_required(login_url=reverse_lazy("admin:login"))
def bestellung_pdf_an_kunden_senden(request, object_id):
    if request.method == "POST":
        if Bestellung.objects.filter(id=int(object_id)).exists():
            if Bestellung.objects.get(id=int(object_id)).send_pdf_rechnung_to_customer():
                messages.success(request, "Rechnung an Kunden gesendet!")
            else:
                messages.error(
                    request, "Rechnung konnte nicht an Kunden gesendet werden!")
        else:
            messages.error(request, "Bestellung konnte nicht gefunden werden!")
        return redirect(reverse("admin:kmuhelper_bestellung_change", args=[object_id]))
    else:
        return render(request, "admin/kmuhelper/_confirm.html", {"action": "Rechnung an Kunden senden"})


# Kunden-Entpunkte

def public_view_order(request, order_id, order_key):
    if Bestellung.objects.filter(id=int(order_id)).exists():
        obj = Bestellung.objects.get(id=int(order_id))
        lieferschein = bool("lieferschein" in dict(request.GET))
        digital = not bool("druck" in dict(request.GET))
        if str(obj.order_key) == order_key:
            return obj.get_pdf(lieferschein=lieferschein, digital=digital)
        else:
            return HttpResponse("Der Bestellungsschlüssel dieser Bestellung ist falsch!")
    else:
        return HttpResponse("Bestellung wurde nicht gefunden")



from django.contrib import admin, messages
from datetime import datetime
from pytz import utc

from kmuhelper.emails.models import EMail

#######

@admin.register(EMail)
class EMailAdmin(admin.ModelAdmin):
    fieldsets = [
        ("Infos", {'fields': ['subject', 'to', 'typ']}),
        ('Inhalt', {'fields': ['html_template', 'html_context']}),
        ('Zeiten', {'fields': ['time_created', 'time_sent']}),
        ('Extra', {'fields': ['data']}),
    ]

    readonly_fields = ('time_created', 'time_sent')

    ordering = ('time_sent', 'time_created',)

    list_display = ('subject', 'to', 'typ', 'time_created', 'time_sent', 'is_sent')
    search_fields = ['subject', 'to']

    def has_module_permission(self, request):
        return {}
    
# 

modeladmins = [
    (EMail, EMailAdmin),
]



from django.http import HttpResponse, FileResponse
from django.conf import settings
from django.core import mail
from django.template.loader import get_template
from django.utils import translation

from datetime import datetime
from io import BytesIO

import sys
import requests

################


def python_version():
    return str(sys.version.split(" ")[0])

def _package_versions_pypi(project, testpypi=False):
    from packaging.version import Version, InvalidVersion
    url = "https://"+("test." if testpypi else "")+"pypi.org/pypi/"+project+"/json"
    versions = requests.get(url).json()["releases"].keys()
    versions_safe = []
    for v in versions:
        try:
            Version(v)
            versions_safe.append(v)
        except (InvalidVersion):
            continue
    return sorted(versions_safe, key=Version)

def package_version(package, testpypi=False):
    from packaging.version import parse as parse_version
    import subprocess

    all_versions = _package_versions_pypi(package, testpypi)

    latest_version = all_versions[-1]

    # cmd = [sys.executable, '-m', 'pip', 'install', '{}==random'.format(package)]

    # if testpypi:
    #     cmd.insert(4, "https://test.pypi.org/simple/")
    #     cmd.insert(4, "-i")
    
    # latest_version = str(subprocess.run(cmd, capture_output=True, text=True))
    # latest_version = latest_version[latest_version.find('(from versions:')+15:]
    # latest_version = latest_version[:latest_version.find(')')]
    # latest_version = latest_version.replace(' ','').split(',')[-1]

    current_version = str(subprocess.run([sys.executable, '-m', 'pip', 'show', '{}'.format(package)], capture_output=True, text=True))
    current_version = current_version[current_version.find('Version:')+8:]
    current_version = current_version[:current_version.find('\\n')].replace(' ', '')
    return {
        #"all":      all_versions,
        "latest":    latest_version, 
        "current":   current_version, 
        "uptodate":  parse_version(latest_version) <= parse_version(current_version),
    }

################

def getfirstindex(data:list, search:list):
    for s in search:
        if s in data:
            return data.index(s)
    return None

def runden(preis):
    return float("{:.2f}".format(float(round(round(preis / 0.05) * 0.05, 2))))

def formatprice(preis):
    return "{:.2f}".format(float(preis))

def clean(string, lang="de"):
    if "[:"+lang+"]" in string:
        return string.split("[:"+lang+"]")[1].split("[:")[0]
    elif "[:de]" in string:
        return string.split("[:de]")[1].split("[:")[0]
    else:
        return string

###############

def send_mail(subject:str, to:str, template_name:str, context:dict={}, **kwargs):
    html_message = get_template("kmuhelper/emails/"+template_name).render(context)

    msg = mail.EmailMessage(
        subject=subject,
        body=html_message,
        to=[to],
        **kwargs
    )

    msg.content_subtype = "html"

    return bool(msg.send())

def send_pdf(subject:str, to:str, template_name:str, pdf:BytesIO, pdf_filename:str="file.pdf", context:dict={}, **kwargs):
    html_message = get_template("kmuhelper/emails/"+template_name).render(context)

    msg = mail.EmailMessage(
        subject=subject,
        body=html_message,
        to=[to],
        **kwargs
    )

    msg.content_subtype = "html"
    msg.attach(filename=pdf_filename, content=pdf.read(), mimetype="application/pdf")

    return bool(msg.send())

###############

def modulo10rekursiv(strNummer):
    intTabelle = [
        [0,9,4,6,8,2,7,1,3,5],
        [9,4,6,8,2,7,1,3,5,0],
        [4,6,8,2,7,1,3,5,0,9],
        [6,8,2,7,1,3,5,0,9,4],
        [8,2,7,1,3,5,0,9,4,6],
        [2,7,1,3,5,0,9,4,6,8],
        [7,1,3,5,0,9,4,6,8,2],
        [1,3,5,0,9,4,6,8,2,7],
        [3,5,0,9,4,6,8,2,7,1],
        [5,0,9,4,6,8,2,7,1,3],
    ]
    strNummer = strNummer.replace(" ","")
    uebertrag = 0
    for num in strNummer:
        uebertrag = intTabelle[uebertrag][int(num)]
    return [0,9,8,7,6,5,4,3,2,1][uebertrag]



from django.urls import path
from django.conf import settings

from kmuhelper.api import views

#######################

urlpatterns = [
    path('errors/not-authenticated', views.not_authenticated, name="api-not-authenticated"),

    path('versions', views.versions, name="api-versions"),
    path('orders/unpaid', views.orders_unpaid, name="api-orders-unpaid"),
]



from django.urls import path
from django.conf import settings

from kmuhelper.emails import views

#######################

urlpatterns = [
    path('<object_id>/view', views.email_view, name="email-view"),
]



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseBadRequest
from django.shortcuts import render, redirect
from django.urls import reverse, reverse_lazy
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt

# Create your views here.

@login_required(login_url=reverse_lazy("admin:login"))
def home(request):
    return render(request, "kmuhelper/home.html", {"has_permission": True})


@login_required(login_url=reverse_lazy("admin:login"))
def admin(request):
    return redirect(reverse("admin:app_list", kwargs={"app_label": "kmuhelper"}))

def _templatetest(request, templatename):
    return render(request, templatename, {})



# pylint: disable=unsupported-assignment-operation

from django.db import models
from django.core import mail
from django.urls import reverse
from django.utils import timezone
from django.utils.html import mark_safe
from django.template.loader import get_template
from django.conf import settings

import uuid
from io import BytesIO

#####

from rich import print

prefix = "[deep_pink4][KMUHelper E-Mails][/] -"

def log(string, *args):
    print(prefix, string, *args)

#####

EMAILTYPEN = [
    ("", "-"),
    ("kunde_registriert", "Registrierungsmail Kunde"),
    ("bestellung_rechnung", "Rechnung"),
    ("bestellung_stock_warning", "Warnung zum Lagerbestand"),
]

#####

class EMailAttachment():
    def __init__(self, filename, content, url=None, mimetype="application/pdf"):
        self.filename = filename
        self.content = content
        self.url = url
        self.mimetype = mimetype

        if isinstance(content, BytesIO):
            self.content = content.read()

class EMail(models.Model):
    typ = models.CharField("Typ", choices=EMAILTYPEN, max_length=50, default="", blank=True)
    
    subject = models.CharField("Betreff", max_length=50)
    to = models.EmailField("Empfänger")

    html_template = models.CharField("Dateiname der Vorlage", max_length=100)
    html_context = models.JSONField("Daten", default=dict)

    token = models.UUIDField("Token", default=uuid.uuid4, editable=False)

    time_created = models.DateTimeField("Erstellt am", auto_now_add=True)
    time_sent = models.DateTimeField("Gesendet um", blank=True, null=True, default=None)

    data = models.JSONField("Extradaten", default=dict)

    notes = models.TextField("Notizen", blank=True, default="")

    def __str__(self):
        return f"{self.subject} ({self.pk})"
    __str__.short_description = "E-Mail"

    def is_sent(self):
        return self.time_sent is not None
    is_sent.short_description = "Gesendet?"
    is_sent.boolean = True

    def render(self, online=False):
        context = dict(self.html_context)
        context["isonline"] = online
        context["view_online_url"] = None if online else self.get_url()
        context["data"] = self.data if online else None
        return get_template("kmuhelper/emails/"+str(self.html_template)).render(context)

    def send(self, attachments=[], **options):
        msg = mail.EmailMessage(
            subject=self.subject,
            body=self.render(),
            to=[self.to],
            **options
        )

        msg.content_subtype = "html"

        for attachment in attachments:
            msg.attach(filename=attachment.filename, content=attachment.content, mimetype=attachment.mimetype)

        success = msg.send()

        log("ID:", self.pk, "- Subject:", self.subject, "- Success:", success)

        if success:
            self.time_sent = timezone.now()
            self.data["attachments"] = [{"filename": a.filename, "url": a.url} for a in attachments]
            self.save()
        return success

    def get_url(self):
        domain = getattr(settings, "KMUHELPER_DOMAIN", None)
        if domain:
            return domain+reverse("kmuhelper:email-view", kwargs={"object_id": self.pk})+f"?token={self.token}"
        else:
            log("Einstellung KMUHELPER_DOMAIN ist nicht gesetzt! E-Mails werden ohne 'online ansehen' Links versendet!")
            return None

    class Meta:
        verbose_name = "E-Mail"
        verbose_name_plural = "E-Mails"

    objects = models.Manager()



# pylint: disable=no-member

from django.urls import path
from django.conf import settings

from kmuhelper.integrations.woocommerce import views

#######################

urlpatterns = [
    path('auth/start', views.wc_auth_start, name="wc-auth-start"),
    path('auth/end', views.wc_auth_end, name="wc-auth-end"),
    path('auth/key', views.wc_auth_key, name="wc-auth-key"),
    path('import/products', views.wc_import_products, name="wc-import-products"),
    path('import/customers', views.wc_import_customers, name="wc-import-customers"),
    path('import/categories', views.wc_import_categories, name="wc-import-categories"),
    path('import/orders', views.wc_import_orders, name="wc-import-orders"),
    path('update/product/<object_id>', views.wc_update_product, name="wc-update-product"),
    path('update/customer/<object_id>', views.wc_update_customer, name="wc-update-customer"),
    path('update/category/<object_id>', views.wc_update_category, name="wc-update-category"),
    path('update/order/<object_id>', views.wc_update_order, name="wc-update-order"),
    
    path('webhooks', views.wc_webhooks, name="wc-webhooks"),
]



from django.contrib import admin, messages
from datetime import datetime
from pytz import utc

from kmuhelper.app.models import ToDoNotiz, ToDoVersand, ToDoZahlungseingang, ToDoLagerbestand, ToDoLieferung
from kmuhelper.main.admin import NotizenAdmin, BestellungsAdmin, LieferungenAdmin, ProduktAdmin
from kmuhelper.main.models import Ansprechpartner, Bestellung, Kategorie, Kosten, Kunde, Lieferant, Lieferung, Notiz, Produkt, Zahlungsempfaenger, Einstellung

from kmuhelper.utils import package_version, python_version
from kmuhelper.integrations.woocommerce import WooCommerce

#######


@admin.register(ToDoNotiz)
class ToDoNotizenAdmin(NotizenAdmin):
    list_editable = ["priority", "erledigt"]
    list_filter = ["priority"]

    ordering = ["-priority", "erstellt_am"]

    def has_module_permission(self, request):
        return {}

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        from django.urls import path
        from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe
        from functools import update_wrapper

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(allow_iframe(view))(*args, **kwargs)
            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
            path('', wrap(self.changelist_view),
                 name='%s_%s_changelist' % info),
            path('add/', wrap(self.add_view), name='%s_%s_add' % info),
            path('autocomplete/', wrap(self.autocomplete_view),
                 name='%s_%s_autocomplete' % info),
            # path('<path:object_id>/history/', wrap(self.history_view), name='%s_%s_history' % info),
            # path('<path:object_id>/delete/', wrap(self.delete_view), name='%s_%s_delete' % info),
            path('<path:object_id>/change/', wrap(self.change_view),
                 name='%s_%s_change' % info),
        ]


@admin.register(ToDoVersand)
class ToDoVersandAdmin(BestellungsAdmin):
    list_display = ('id', 'info', 'trackingnummer',
                    'versendet', 'status', 'html_todo_notiz')
    list_editable = ("trackingnummer", "versendet", "status")
    list_filter = ('status', 'bezahlt')

    ordering = ("bezahlt", "-datum")

    actions = ()

    def has_module_permission(self, request):
        return {}

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        from django.urls import path
        from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe
        from functools import update_wrapper

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(allow_iframe(view))(*args, **kwargs)
            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
            path('', wrap(self.changelist_view),
                 name='%s_%s_changelist' % info),
            path('add/', wrap(self.add_view), name='%s_%s_add' % info),
            path('autocomplete/', wrap(self.autocomplete_view),
                 name='%s_%s_autocomplete' % info),
            # path('<path:object_id>/history/', wrap(self.history_view), name='%s_%s_history' % info),
            # path('<path:object_id>/delete/', wrap(self.delete_view), name='%s_%s_delete' % info),
            path('<path:object_id>/change/', wrap(self.change_view),
                 name='%s_%s_change' % info),
        ]


@admin.register(ToDoZahlungseingang)
class ToDoZahlungseingangAdmin(BestellungsAdmin):
    list_display = ('id', 'info', 'bezahlt', 'status',
                    'fix_summe', 'html_todo_notiz')
    list_editable = ("bezahlt", "status")
    list_filter = ('status', 'versendet', 'zahlungsmethode')

    ordering = ("versendet", "-datum")

    actions = ()

    def has_module_permission(self, request):
        return {}

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        from django.urls import path
        from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe
        from functools import update_wrapper

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(allow_iframe(view))(*args, **kwargs)
            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
            path('', wrap(self.changelist_view),
                 name='%s_%s_changelist' % info),
            path('add/', wrap(self.add_view), name='%s_%s_add' % info),
            path('autocomplete/', wrap(self.autocomplete_view),
                 name='%s_%s_autocomplete' % info),
            # path('<path:object_id>/history/', wrap(self.history_view), name='%s_%s_history' % info),
            # path('<path:object_id>/delete/', wrap(self.delete_view), name='%s_%s_delete' % info),
            path('<path:object_id>/change/', wrap(self.change_view),
                 name='%s_%s_change' % info),
        ]


@admin.register(ToDoLagerbestand)
class ToDoLagerbestandAdmin(ProduktAdmin):
    list_display = ('nr', 'clean_name', 'lagerbestand',
                    'preis', 'bemerkung', 'html_todo_notiz')
    list_editable = ["lagerbestand"]

    actions = ["lagerbestand_zuruecksetzen"]

    def has_module_permission(self, request):
        return {}

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        from django.urls import path
        from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe
        from functools import update_wrapper

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(allow_iframe(view))(*args, **kwargs)
            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
            path('', wrap(self.changelist_view),
                 name='%s_%s_changelist' % info),
            path('add/', wrap(self.add_view), name='%s_%s_add' % info),
            path('autocomplete/', wrap(self.autocomplete_view),
                 name='%s_%s_autocomplete' % info),
            # path('<path:object_id>/history/', wrap(self.history_view), name='%s_%s_history' % info),
            # path('<path:object_id>/delete/', wrap(self.delete_view), name='%s_%s_delete' % info),
            path('<path:object_id>/change/', wrap(self.change_view),
                 name='%s_%s_change' % info),
        ]


@admin.register(ToDoLieferung)
class ToDoLieferungenAdmin(LieferungenAdmin):
    list_display = ('name', 'datum', 'anzahlprodukte', 'html_todo_notiz')
    list_filter = ()

    def has_module_permission(self, request):
        return {}

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        from django.urls import path
        from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe
        from functools import update_wrapper

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(allow_iframe(view))(*args, **kwargs)
            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
            path('', wrap(self.changelist_view),
                 name='%s_%s_changelist' % info),
            path('add/', wrap(self.add_view), name='%s_%s_add' % info),
            path('autocomplete/', wrap(self.autocomplete_view),
                 name='%s_%s_autocomplete' % info),
            # path('<path:object_id>/history/', wrap(self.history_view), name='%s_%s_history' % info),
            # path('<path:object_id>/delete/', wrap(self.delete_view), name='%s_%s_delete' % info),
            path('<path:object_id>/change/', wrap(self.change_view),
                 name='%s_%s_change' % info),
        ]

# 

modeladmins = [
    (ToDoNotiz, ToDoNotizenAdmin),
    (ToDoVersand, ToDoVersandAdmin),
    (ToDoZahlungseingang, ToDoZahlungseingangAdmin),
    (ToDoLagerbestand, ToDoLagerbestandAdmin),
    (ToDoLieferung, ToDoLieferungenAdmin),
]

